@@ -68,10 +68,11 @@ from xonsh.tools import (
     is_logfile_opt,
     is_nonstring_seq_of_strings,
     is_path,
-    is_str_str_dict,
+    is_regex,
     is_string,
     is_string_or_callable,
     is_string_set,
+    is_tok_color_dict,
     is_valid_shlvl,
     logfile_opt_to_str,
     path_to_str,
@@ -96,7 +97,7 @@ from xonsh.tools import (
     to_logfile_opt,
     to_repr_pretty_,
     to_shlvl,
-    to_str_str_dict,
+    to_tok_color_dict,
 )
 
 events.doc(
@@ -110,7 +111,6 @@ cause a recursion until the limit.
 """,
 )
 
-
 events.doc(
     "on_envvar_change",
     """
@@ -122,7 +122,6 @@ cause a recursion until the limit.
 """,
 )
 
-
 events.doc(
     "on_pre_spec_run_ls",
     """
@@ -133,7 +132,6 @@ command.
 """,
 )
 
-
 events.doc(
     "on_lscolors_change",
     """
@@ -182,7 +180,7 @@ def locale_convert(key):
             val = locale.setlocale(LOCALE_CATS[key])
         except (locale.Error, KeyError):
             msg = f"Failed to set locale {key!r} to {val!r}"
-            warnings.warn(msg, RuntimeWarning)
+            warnings.warn(msg, RuntimeWarning, stacklevel=2)
         return val
 
     return lc_converter
@@ -569,6 +567,14 @@ def xonsh_data_dir(env):
     return xdd
 
 
+@default_value
+def xonsh_cache_dir(env):
+    """Ensures and returns the $XONSH_CACHE_DIR"""
+    xdd = os.path.expanduser(os.path.join(env.get("XDG_CACHE_HOME"), "xonsh"))
+    os.makedirs(xdd, exist_ok=True)
+    return xdd
+
+
 @default_value
 def xonsh_config_dir(env):
     """``$XDG_CONFIG_HOME/xonsh``"""
@@ -851,15 +857,21 @@ class GeneralSetting(Xettings):
         "will print information about how to continue the stopped process.",
     )
 
-    COMMANDS_CACHE_SIZE_WARNING = Var.with_default(
-        6000,
-        "Number of files on the PATH above which a warning is shown.",
-    )
     COMMANDS_CACHE_SAVE_INTERMEDIATE = Var.with_default(
         False,
         "If enabled, the CommandsCache is saved between runs and can reduce the startup time.",
     )
 
+    ENABLE_COMMANDS_CACHE = Var(
+        default=True,
+        doc="command names in a directory are cached when enabled.",
+        doc_default="True"
+        "On some platforms it may not be accurate enough"
+        "(e.g. Windows, Linux save mtime in seconds). "
+        "setting it to False would disable the caching mechanism "
+        "and may slow down the shell",
+    )
+
     HOSTNAME = Var.with_default(
         default=default_value(lambda env: platform.node()),
         doc="Automatically set to the name of the current host.",
@@ -1011,6 +1023,12 @@ class GeneralSetting(Xettings):
         doc_default="``~/.local/share``",
         type_str="str",
     )
+    XDG_CACHE_HOME = Var.with_default(
+        os.path.expanduser(os.path.join("~", ".cache")),
+        "The base directory relative to which user-specific non-essential data files should be stored.",
+        doc_default="``~/.cache``",
+        type_str="str",
+    )
     XDG_DATA_DIRS = Var.with_default(
         xdg_data_dirs,
         "A list of directories where system level data files are stored.",
@@ -1099,6 +1117,12 @@ The file should contain a function with the signature
         doc_default="``$XDG_DATA_HOME/xonsh``",
         type_str="str",
     )
+    XONSH_CACHE_DIR = Var.with_default(
+        xonsh_cache_dir,
+        "This is the location where cache files used by xonsh are stored, such as commands-cache...",
+        doc_default="``$XDG_CACHE_HOME/xonsh``",
+        type_str="str",
+    )
     XONSH_ENCODING = Var.with_default(
         DEFAULT_ENCODING,
         "This is the encoding that xonsh should use for subprocess operations.",
@@ -1150,8 +1174,8 @@ The file should contain a function with the signature
         "``!()`` and ``![]`` operators.",
     )
     XONSH_STYLE_OVERRIDES = Var(
-        is_str_str_dict,
-        to_str_str_dict,
+        is_tok_color_dict,
+        to_tok_color_dict,
         dict_to_str,
         {},
         "A dictionary containing custom prompt_toolkit/pygments style definitions.\n"
@@ -1362,6 +1386,30 @@ class PromptSetting(Xettings):
         "Prompt text for 2nd+ lines of input, may be str or function which "
         "returns a str.",
     )
+    MULTILINE_PROMPT_PRE = Var(
+        is_string_or_callable,
+        ensure_string,
+        ensure_string,
+        "",
+        "Indicator inserted before the line continuation marks set "
+        "in ``$MULTILINE_PROMPT``. Can be used to mark the start of "
+        "a semantic continuation prompt "
+        "(see `Semantic Prompts <https://gitlab.freedesktop.org/Per_Bothner/specifications/blob/master/proposals/semantic-prompts.md>`_ "
+        "or `WezTerm <https://wezfurlong.org/wezterm/shell-integration.html>`_ "
+        "for more details). May be str or function which returns a str.",
+    )
+    MULTILINE_PROMPT_POS = Var(
+        is_string_or_callable,
+        ensure_string,
+        ensure_string,
+        "",
+        "Indicator inserted after the line continuation marks set "
+        "in ``$MULTILINE_PROMPT``. Can be used to mark the end of "
+        "a semantic continuation prompt and the beginning of user input "
+        "(see `Semantic Prompts <https://gitlab.freedesktop.org/Per_Bothner/specifications/blob/master/proposals/semantic-prompts.md>`_ "
+        "or `WezTerm <https://wezfurlong.org/wezterm/shell-integration.html>`_ "
+        "for more details). May be str or function which returns a str.",
+    )
     PRETTY_PRINT_RESULTS = Var.with_default(
         True,
         'Flag for "pretty printing" return values.',
@@ -1580,6 +1628,14 @@ class PromptHistorySetting(Xettings):
         "Save current working directory to the history.",
         doc_default="True",
     )
+    XONSH_HISTORY_IGNORE_REGEX = Var(
+        is_regex,
+        to_itself,
+        ensure_string,
+        None,
+        "Set a filter criteria for history items using a regular expression. "
+        "Any matching items will not be retained in the history.",
+    )
 
 
 class PTKSetting(PromptSetting):  # sub-classing -> sub-group
@@ -1616,8 +1672,8 @@ class PTKSetting(PromptSetting):  # sub-classing -> sub-group
         "colors. Default is an empty string which means that prompt toolkit decide.",
     )
     PTK_STYLE_OVERRIDES = Var(
-        is_str_str_dict,
-        to_str_str_dict,
+        is_tok_color_dict,
+        to_tok_color_dict,
         dict_to_str,
         {},
         "A dictionary containing custom prompt_toolkit style definitions. (deprecated)",
@@ -1892,7 +1948,12 @@ class Env(cabc.MutableMapping):
             self._d["PATH"] = list(PATH_DEFAULT)
         self._detyped = None
 
+    def get_detyped(self, key: str):
+        detyped = self.detype()
+        return detyped.get(key)
+
     def detype(self):
+        """return a dict that can be used as ``os.environ``"""
         if self._detyped is not None:
             return self._detyped
         ctx = {}
