@@ -1,21 +1,18 @@
-# -*- coding: utf-8 -*-
 """Tools to help interface with foreign shells, such as Bash."""
+import collections.abc as cabc
+import functools
 import os
 import re
-import json
 import shlex
+import subprocess
 import sys
 import tempfile
-import builtins
-import subprocess
 import warnings
-import functools
-import collections.abc as cabc
 
+from xonsh.built_ins import XSH
 from xonsh.lazyasd import lazyobject
-from xonsh.tools import to_bool, ensure_string
-from xonsh.platform import ON_WINDOWS, ON_CYGWIN, ON_MSYS
-
+from xonsh.platform import ON_CYGWIN, ON_MSYS, ON_WINDOWS
+from xonsh.tools import ensure_string, to_bool
 
 COMMAND = """{seterrprevcmd}
 {prevcmd}
@@ -31,62 +28,21 @@ echo __XONSH_FUNCS_END__
 {postcmd}
 {seterrpostcmd}"""
 
-DEFAULT_BASH_FUNCSCMD = r"""# get function names from declare
-declstr=$(declare -F)
+DEFAULT_BASH_FUNCSCMD = """# get function names from declare
+declstr=$(echo $(declare -F))
 read -r -a decls <<< $declstr
-funcnames=""
 for((n=0;n<${#decls[@]};n++)); do
   if (( $(($n % 3 )) == 2 )); then
-    # get every 3rd entry
-    funcnames="$funcnames ${decls[$n]}"
+    echo -n "${decls[$n]} "
   fi
 done
-
-# get functions locations: funcname lineno filename
-shopt -s extdebug
-namelocfilestr=$(declare -F $funcnames)
-shopt -u extdebug
-
-# print just names and files as JSON object
-read -r -a namelocfile <<< $namelocfilestr
-sep=" "
-namefile="{"
-while IFS='' read -r line || [[ -n "$line" ]]; do
-  name=${line%%"$sep"*}
-  locfile=${line#*"$sep"}
-  loc=${locfile%%"$sep"*}
-  file=${locfile#*"$sep"}
-  namefile="${namefile}\"${name}\":\"${file//\\/\\\\}\","
-done <<< "$namelocfilestr"
-if [[ "{" == "${namefile}" ]]; then
-  namefile="${namefile}}"
-else
-  namefile="${namefile%?}}"
-fi
-echo $namefile"""
+echo"""
 
 DEFAULT_ZSH_FUNCSCMD = """# get function names
-autoload -U is-at-least  # We'll need to version check zsh
-namefile="{"
 for name in ${(ok)functions}; do
-  # force zsh to load the func in order to get the filename,
-  # but use +X so that it isn't executed.
-  autoload +X $name || continue
-  loc=$(whence -v $name)
-  loc=${(z)loc}
-  if is-at-least 5.2; then
-    file=${loc[-1]}
-  else
-    file=${loc[7,-1]}
-  fi
-  namefile="${namefile}\\"${name}\\":\\"${(Q)file:A}\\","
+  echo -n "$name "
 done
-if [[ "{" == "${namefile}" ]]; then
-  namefile="${namefile}}"
-else
-  namefile="${namefile%?}}"
-fi
-echo ${namefile}"""
+echo"""
 
 
 # mapping of shell name aliases to keys in other lookup dictionaries.
@@ -143,7 +99,7 @@ def DEFAULT_SETERRPOSTCMD():
     return {"bash": "", "zsh": "", "cmd": "if errorlevel 1 exit 1"}
 
 
-@functools.lru_cache()
+@functools.lru_cache
 def foreign_shell_data(
     shell,
     interactive=True,
@@ -164,6 +120,7 @@ def foreign_shell_data(
     seterrpostcmd=None,
     show=False,
     dryrun=False,
+    files=(),
 ):
     """Extracts data from a foreign (non-xonsh) shells. Currently this gets
     the environment, aliases, and functions but may be extended in the future.
@@ -225,7 +182,8 @@ def foreign_shell_data(
         Whether or not to display the script that will be run.
     dryrun : bool, optional
         Whether or not to actually run and process the command.
-
+    files : tuple of str, optional
+        Paths to source.
 
     Returns
     -------
@@ -276,8 +234,8 @@ def foreign_shell_data(
         tmpfile.write(command.encode("utf8"))
         tmpfile.close()
         cmd.append(tmpfile.name)
-    if currenv is None and hasattr(builtins.__xonsh__, "env"):
-        currenv = builtins.__xonsh__.env.detype()
+    if currenv is None and XSH.env:
+        currenv = XSH.env.detype()
     elif currenv is not None:
         currenv = dict(currenv)
     try:
@@ -288,7 +246,7 @@ def foreign_shell_data(
             # start new session to avoid hangs
             # (doesn't work on Cygwin though)
             start_new_session=((not ON_CYGWIN) and (not ON_MSYS)),
-            universal_newlines=True,
+            text=True,
         )
     except (subprocess.CalledProcessError, FileNotFoundError):
         if not safe:
@@ -298,8 +256,20 @@ def foreign_shell_data(
         if use_tmpfile:
             os.remove(tmpfile.name)
     env = parse_env(s)
-    aliases = parse_aliases(s)
-    funcs = parse_funcs(s, shell=shell, sourcer=sourcer, extra_args=extra_args)
+    aliases = parse_aliases(
+        s,
+        shell=shell,
+        sourcer=sourcer,
+        files=files,
+        extra_args=extra_args,
+    )
+    funcs = parse_funcs(
+        s,
+        shell=shell,
+        sourcer=sourcer,
+        files=files,
+        extra_args=extra_args,
+    )
     aliases.update(funcs)
     return env, aliases
 
@@ -332,7 +302,12 @@ def ALIAS_RE():
     )
 
 
-def parse_aliases(s):
+@lazyobject
+def FS_EXEC_ALIAS_RE():
+    return re.compile(r";|`|\$\(")
+
+
+def parse_aliases(s, shell, sourcer=None, files=(), extra_args=()):
     """Parses the aliases portion of string into a dict."""
     m = ALIAS_RE.search(s)
     if m is None:
@@ -352,10 +327,24 @@ def parse_aliases(s):
             # strip one single quote at the start and end of value
             if value[0] == "'" and value[-1] == "'":
                 value = value[1:-1]
-            value = shlex.split(value)
+            # now compute actual alias
+            if FS_EXEC_ALIAS_RE.search(value) is None:
+                # simple list of args alias
+                value = shlex.split(value)
+            else:
+                # alias is more complex, use ExecAlias, but via shell
+                value = ForeignShellExecAlias(
+                    src=value,
+                    shell=shell,
+                    sourcer=sourcer,
+                    files=files,
+                    extra_args=extra_args,
+                )
         except ValueError as exc:
             warnings.warn(
-                'could not parse alias "{0}": {1!r}'.format(key, exc), RuntimeWarning
+                f'could not parse alias "{key}": {exc!r}',
+                RuntimeWarning,
+                stacklevel=2,
             )
             continue
         aliases[key] = value
@@ -369,7 +358,7 @@ def FUNCS_RE():
     )
 
 
-def parse_funcs(s, shell, sourcer=None, extra_args=()):
+def parse_funcs(s, shell, sourcer=None, files=(), extra_args=()):
     """Parses the funcs portion of a string into a dict of callable foreign
     function wrappers.
     """
@@ -379,93 +368,68 @@ def parse_funcs(s, shell, sourcer=None, extra_args=()):
     g1 = m.group(1)
     if ON_WINDOWS:
         g1 = g1.replace(os.sep, os.altsep)
-    try:
-        namefiles = json.loads(g1.strip())
-    except json.decoder.JSONDecodeError as exc:
-        msg = (
-            "{0!r}\n\ncould not parse {1} functions:\n"
-            "  s  = {2!r}\n"
-            "  g1 = {3!r}\n\n"
-            "Note: you may be seeing this error if you use zsh with "
-            "prezto. Prezto overwrites GNU coreutils functions (like echo) "
-            "with its own zsh functions. Please try disabling prezto."
-        )
-        warnings.warn(msg.format(exc, shell, s, g1), RuntimeWarning)
-        return {}
-    sourcer = DEFAULT_SOURCERS.get(shell, "source") if sourcer is None else sourcer
+    funcnames = g1.split()
     funcs = {}
-    for funcname, filename in namefiles.items():
-        if funcname.startswith("_") or not filename:
+    for funcname in funcnames:
+        if funcname.startswith("_"):
             continue  # skip private functions and invalid files
-        if not os.path.isabs(filename):
-            filename = os.path.abspath(filename)
         wrapper = ForeignShellFunctionAlias(
-            name=funcname,
+            funcname=funcname,
             shell=shell,
             sourcer=sourcer,
-            filename=filename,
+            files=files,
             extra_args=extra_args,
         )
         funcs[funcname] = wrapper
     return funcs
 
 
-class ForeignShellFunctionAlias(object):
+class ForeignShellBaseAlias:
     """This class is responsible for calling foreign shell functions as if
     they were aliases. This does not currently support taking stdin.
     """
 
-    INPUT = '{sourcer} "{filename}"\n' "{funcname} {args}\n"
+    INPUT = "echo ForeignShellBaseAlias {shell} {args}\n"
 
-    def __init__(self, name, shell, filename, sourcer=None, extra_args=()):
+    def __init__(self, shell, sourcer=None, files=(), extra_args=()):
         """
         Parameters
         ----------
-        name : str
-            function name
         shell : str
             Name or path to shell
-        filename : str
-            Where the function is defined, path to source.
         sourcer : str or None, optional
             Command to source foreign files with.
+        files : tuple of str, optional
+            Paths to source.
         extra_args : tuple of str, optional
             Additional command line options to pass into the shell.
         """
         sourcer = DEFAULT_SOURCERS.get(shell, "source") if sourcer is None else sourcer
-        self.name = name
         self.shell = shell
-        self.filename = filename
         self.sourcer = sourcer
+        self.files = files
         self.extra_args = extra_args
 
-    def __eq__(self, other):
-        if (
-            not hasattr(other, "name")
-            or not hasattr(other, "shell")
-            or not hasattr(other, "filename")
-            or not hasattr(other, "sourcer")
-            or not hasattr(other, "exta_args")
-        ):
-            return NotImplemented
-        return (
-            (self.name == other.name)
-            and (self.shell == other.shell)
-            and (self.filename == other.filename)
-            and (self.sourcer == other.sourcer)
-            and (self.extra_args == other.extra_args)
-        )
+    def _input_kwargs(self):
+        return {
+            "shell": self.shell,
+            "extra_args": self.extra_args,
+        }
 
-    def __call__(self, args, stdin=None):
+    def __eq__(self, other):
+        if not hasattr(other, "_input_kwargs") or not callable(other._input_kwargs):
+            return NotImplemented
+        return self._input_kwargs() == other._input_kwargs()
+
+    def __call__(
+        self, args, stdin=None, stdout=None, stderr=None, spec=None, stack=None
+    ):
         args, streaming = self._is_streaming(args)
-        input = self.INPUT.format(
-            sourcer=self.sourcer,
-            filename=self.filename,
-            funcname=self.name,
-            args=" ".join(args),
-        )
-        cmd = [self.shell] + list(self.extra_args) + ["-c", input]
-        env = builtins.__xonsh__.env
+        input = self.INPUT.format(args=" ".join(args), **self._input_kwargs())
+        if len(self.files) > 0:
+            input = "".join([f'{self.sourcer} "{f}"\n' for f in self.files]) + input
+        cmd = [self.shell] + list(self.extra_args) + ["-ic", input]
+        env = XSH.env
         denv = env.detype()
         if streaming:
             subprocess.check_call(cmd, env=denv)
@@ -479,13 +443,93 @@ class ForeignShellFunctionAlias(object):
             out = out.replace("\r\n", "\n")
         return out
 
-    def _is_streaming(self, args):
+    def __repr__(self):
+        return (
+            self.__class__.__name__
+            + "("
+            + ", ".join([f"{k}={v!r}" for k, v in sorted(self._input_kwargs().items())])
+            + ")"
+        )
+
+    @staticmethod
+    def _is_streaming(args):
         """Test and modify args if --xonsh-stream is present."""
-        if "--xonsh-stream" not in args:
-            return args, False
+        if "--xonsh-nostream" not in args:
+            return args, True
         args = list(args)
-        args.remove("--xonsh-stream")
-        return args, True
+        args.remove("--xonsh-nostream")
+        return args, False
+
+
+class ForeignShellFunctionAlias(ForeignShellBaseAlias):
+    """This class is responsible for calling foreign shell functions as if
+    they were aliases. This does not currently support taking stdin.
+    """
+
+    INPUT = "{funcname} {args}\n"
+
+    def __init__(self, funcname, shell, sourcer=None, files=(), extra_args=()):
+        """
+        Parameters
+        ----------
+        funcname : str
+            function name
+        shell : str
+            Name or path to shell
+        sourcer : str or None, optional
+            Command to source foreign files with.
+        files : tuple of str, optional
+            Paths to source.
+        extra_args : tuple of str, optional
+            Additional command line options to pass into the shell.
+        """
+        super().__init__(
+            shell=shell, sourcer=sourcer, files=files, extra_args=extra_args
+        )
+        self.funcname = funcname
+
+    def _input_kwargs(self):
+        inp = super()._input_kwargs()
+        inp["funcname"] = self.funcname
+        return inp
+
+
+class ForeignShellExecAlias(ForeignShellBaseAlias):
+    """Provides a callable alias for source code in a foreign shell."""
+
+    INPUT = "{src} {args}\n"
+
+    def __init__(
+        self,
+        src,
+        shell,
+        sourcer=None,
+        files=(),
+        extra_args=(),
+    ):
+        """
+        Parameters
+        ----------
+        src : str
+            Source code in the shell language
+        shell : str
+            Name or path to shell
+        sourcer : str or None, optional
+            Command to source foreign files with.
+        files : tuple of str, optional
+            Paths to source.
+        extra_args : tuple of str, optional
+            Additional command line options to pass into the shell.
+        """
+        super().__init__(
+            shell=shell, sourcer=sourcer, files=files, extra_args=extra_args
+        )
+        self.src = src.strip()
+
+    def _input_kwargs(self):
+        inp = super()._input_kwargs()
+        inp["src"] = self.src
+        return inp
 
 
 @lazyobject
@@ -514,8 +558,7 @@ def ensure_shell(shell):
         shell = dict(shell)
     shell_keys = set(shell.keys())
     if not (shell_keys <= VALID_SHELL_PARAMS):
-        msg = "unknown shell keys: {0}"
-        raise KeyError(msg.format(shell_keys - VALID_SHELL_PARAMS))
+        raise KeyError(f"unknown shell keys: {shell_keys - VALID_SHELL_PARAMS}")
     shell["shell"] = ensure_string(shell["shell"]).lower()
     if "interactive" in shell_keys:
         shell["interactive"] = to_bool(shell["interactive"])
@@ -534,9 +577,9 @@ def ensure_shell(shell):
     if "currenv" in shell_keys and not isinstance(shell["currenv"], tuple):
         ce = shell["currenv"]
         if isinstance(ce, cabc.Mapping):
-            ce = tuple([(ensure_string(k), v) for k, v in ce.items()])
+            ce = tuple((ensure_string(k), v) for k, v in ce.items())
         elif isinstance(ce, cabc.Sequence):
-            ce = tuple([(ensure_string(k), v) for k, v in ce])
+            ce = tuple((ensure_string(k), v) for k, v in ce)
         else:
             raise RuntimeError("unrecognized type for currenv")
         shell["currenv"] = ce
@@ -607,19 +650,18 @@ def load_foreign_aliases(shells):
         A dictionary of the merged aliases.
     """
     aliases = {}
-    xonsh_aliases = builtins.aliases
+    xonsh_aliases = XSH.aliases
     for shell in shells:
         shell = ensure_shell(shell)
         _, shaliases = foreign_shell_data(**shell)
-        if not builtins.__xonsh__.env.get("FOREIGN_ALIASES_OVERRIDE"):
+        if not XSH.env.get("FOREIGN_ALIASES_OVERRIDE"):
             shaliases = {} if shaliases is None else shaliases
             for alias in set(shaliases) & set(xonsh_aliases):
                 del shaliases[alias]
-                if builtins.__xonsh__.env.get("XONSH_DEBUG") > 1:
+                if XSH.env.get("XONSH_DEBUG") >= 1:
                     print(
-                        "aliases: ignoring alias {!r} of shell {!r} "
-                        "which tries to override xonsh alias."
-                        "".format(alias, shell["shell"]),
+                        f"aliases: ignoring alias {alias!r} of shell {shell['shell']!r} "
+                        "which tries to override xonsh alias.",
                         file=sys.stderr,
                     )
         aliases.update(shaliases)
