@@ -180,12 +180,12 @@ def safe_open(fname, mode, buffering=-1):
     # file descriptors
     try:
         return open(fname, mode, buffering=buffering)
-    except PermissionError:
-        raise xt.XonshError(f"xonsh: {fname}: permission denied")
-    except FileNotFoundError:
-        raise xt.XonshError(f"xonsh: {fname}: no such file or directory")
-    except Exception:
-        raise xt.XonshError(f"xonsh: {fname}: unable to open file")
+    except PermissionError as ex:
+        raise xt.XonshError(f"xonsh: {fname}: permission denied") from ex
+    except FileNotFoundError as ex:
+        raise xt.XonshError(f"xonsh: {fname}: no such file or directory") from ex
+    except Exception as ex:
+        raise xt.XonshError(f"xonsh: {fname}: unable to open file") from ex
 
 
 def safe_close(x):
@@ -471,22 +471,25 @@ class SubprocSpec:
             else:
                 cmd = self.cmd
             p = self.cls(cmd, bufsize=bufsize, **kwargs)
-        except PermissionError:
+        except PermissionError as ex:
             e = "xonsh: subprocess mode: permission denied: {0}"
-            raise xt.XonshError(e.format(self.cmd[0]))
-        except FileNotFoundError:
+            raise xt.XonshError(e.format(self.cmd[0])) from ex
+        except FileNotFoundError as ex:
             cmd0 = self.cmd[0]
             if len(self.cmd) == 1 and cmd0.endswith("?"):
                 with contextlib.suppress(OSError):
                     return self.cls(
                         ["man", cmd0.rstrip("?")], bufsize=bufsize, **kwargs
                     )
-            e = f"xonsh: subprocess mode: command not found: {cmd0}"
+            e = f"xonsh: subprocess mode: command not found: {repr(cmd0)}"
             env = XSH.env
             sug = xt.suggest_commands(cmd0, env)
             if len(sug.strip()) > 0:
                 e += "\n" + xt.suggest_commands(cmd0, env)
-            raise xt.XonshError(e)
+            if XSH.env.get("XONSH_INTERACTIVE"):
+                events = XSH.builtins.events
+                events.on_command_not_found.fire(cmd=self.cmd)
+            raise xt.XonshError(e) from ex
         return p
 
     def prep_env_subproc(self, kwargs):
@@ -665,9 +668,9 @@ class SubprocSpec:
             scriptcmd = get_script_subproc_command(self.binary_loc, self.cmd[1:])
             if scriptcmd is not None:
                 self.cmd = scriptcmd
-        except PermissionError:
+        except PermissionError as ex:
             e = "xonsh: subprocess mode: permission denied: {0}"
-            raise xt.XonshError(e.format(self.cmd[0]))
+            raise xt.XonshError(e.format(self.cmd[0])) from ex
 
     def resolve_alias_cls(self):
         """Determine which proxy class to run an alias with."""
@@ -737,7 +740,9 @@ def _update_last_spec(last):
         return
     callable_alias = callable(last.alias)
     if callable_alias:
-        pass
+        if last.cls is ProcProxy and captured == "hiddenobject":
+            # a ProcProxy run using ![] should not be captured
+            return
     else:
         cmds_cache = XSH.commands_cache
         thable = (
