@@ -14,14 +14,15 @@ Original file credits:
                   'Michael Foord')
 """
 
-import re
-import io
-import sys
-import codecs
 import builtins
-import itertools
+import codecs
 import collections
+import io
+import itertools
+import re
+import sys
 import token
+import typing as tp
 from token import (
     AMPER,
     AMPEREQUAL,
@@ -52,11 +53,11 @@ from token import (
     LSQB,
     MINEQUAL,
     MINUS,
+    N_TOKENS,
     NAME,
     NEWLINE,
     NOTEQUAL,
     NUMBER,
-    N_TOKENS,
     OP,
     PERCENT,
     PERCENTEQUAL,
@@ -84,7 +85,7 @@ from xonsh.platform import PYTHON_VERSION_INFO
 
 HAS_WALRUS = PYTHON_VERSION_INFO > (3, 8)
 if HAS_WALRUS:
-    from token import COLONEQUAL
+    from token import COLONEQUAL  # type:ignore
 
 cookie_re = LazyObject(
     lambda: re.compile(r"^[ \t\f]*#.*coding[:=][ \t]*([-\w.]+)", re.ASCII),
@@ -92,14 +93,14 @@ cookie_re = LazyObject(
     "cookie_re",
 )
 blank_re = LazyObject(
-    lambda: re.compile(br"^[ \t\f]*(?:[#\r\n]|$)", re.ASCII), globals(), "blank_re"
+    lambda: re.compile(rb"^[ \t\f]*(?:[#\r\n]|$)", re.ASCII), globals(), "blank_re"
 )
 
 #
 # token modifications
 #
 tok_name = tok_name.copy()
-__all__ = token.__all__ + [
+__all__ = token.__all__ + [  # type:ignore
     "COMMENT",
     "tokenize",
     "detect_encoding",
@@ -113,14 +114,16 @@ __all__ = token.__all__ + [
     "ATEQUAL",
     "DOLLARNAME",
     "IOREDIRECT",
+    "MATCH",
+    "CASE",
 ]
 HAS_ASYNC = PYTHON_VERSION_INFO < (3, 7, 0)
 if HAS_ASYNC:
-    ASYNC = token.ASYNC
-    AWAIT = token.AWAIT
+    ASYNC = token.ASYNC  # type:ignore
+    AWAIT = token.AWAIT  # type:ignore
     ADDSPACE_TOKS = (NAME, NUMBER, ASYNC, AWAIT)
 else:
-    ADDSPACE_TOKS = (NAME, NUMBER)
+    ADDSPACE_TOKS = (NAME, NUMBER)  # type:ignore
 del token  # must clean up token
 
 if HAS_WALRUS:
@@ -128,7 +131,6 @@ if HAS_WALRUS:
 else:
     AUGASSIGN_OPS = r"[+\-*/%&@|^=<>]=?"
 
-
 COMMENT = N_TOKENS
 tok_name[COMMENT] = "COMMENT"
 NL = N_TOKENS + 1
@@ -151,6 +153,12 @@ N_TOKENS += 1
 ATEQUAL = N_TOKENS
 tok_name[N_TOKENS] = "ATEQUAL"
 N_TOKENS += 1
+MATCH = N_TOKENS
+tok_name[N_TOKENS] = "MATCH"
+N_TOKENS += 1
+CASE = N_TOKENS
+tok_name[N_TOKENS] = "CASE"
+N_TOKENS += 1
 _xonsh_tokens = {
     "?": "QUESTION",
     "@=": "ATEQUAL",
@@ -165,6 +173,8 @@ _xonsh_tokens = {
     "${": "DOLLARLBRACE",
     "??": "DOUBLEQUESTION",
     "@$(": "ATDOLLARLPAREN",
+    "match": "MATCH",
+    "case": "CASE",
 }
 
 additional_parenlevs = frozenset({"@(", "!(", "![", "$(", "$[", "${", "@$("})
@@ -177,7 +187,7 @@ for v in _xonsh_tokens.values():
     __all__.append(v)
 del _glbs, v
 
-EXACT_TOKEN_TYPES = {
+EXACT_TOKEN_TYPES: tp.Dict[str, tp.Union[str, int]] = {
     "(": LPAR,
     ")": RPAR,
     "[": LSQB,
@@ -232,8 +242,9 @@ class TokenInfo(collections.namedtuple("TokenInfo", "type string start end line"
     def __repr__(self):
         annotated_type = "%d (%s)" % (self.type, tok_name[self.type])
         return (
-            "TokenInfo(type=%s, string=%r, start=%r, end=%r, line=%r)"
-            % self._replace(type=annotated_type)
+            "TokenInfo(type={}, string={!r}, start={!r}, end={!r}, line={!r})".format(
+                *self._replace(type=annotated_type)
+            )
         )
 
     @property
@@ -295,7 +306,7 @@ String = group(
 )
 
 # Xonsh-specific Syntax
-SearchPath = r"((?:[rgp]+|@\w*)?)`([^\n`\\]*(?:\\.[^\n`\\]*)*)`"
+SearchPath = r"((?:[rgpf]+|@\w*)?)`([^\n`\\]*(?:\\.[^\n`\\]*)*)`"
 
 # Because of leftmost-then-longest match semantics, be sure to put the
 # longest operators first (e.g., if = came before ==, == would get
@@ -329,11 +340,13 @@ _redir_map = (
     "o>2",
     "1>2",
 )
-IORedirect = group(group(*_redir_map), "{}>>?".format(group(*_redir_names)))
-_redir_check = set(_redir_map)
-_redir_check = {"{}>".format(i) for i in _redir_names}.union(_redir_check)
-_redir_check = {"{}>>".format(i) for i in _redir_names}.union(_redir_check)
-_redir_check = frozenset(_redir_check)
+IORedirect = group(group(*_redir_map), f"{group(*_redir_names)}>>?")
+
+_redir_check_0 = set(_redir_map)
+_redir_check_1 = {f"{i}>" for i in _redir_names}.union(_redir_check_0)
+_redir_check_2 = {f"{i}>>" for i in _redir_names}.union(_redir_check_1)
+_redir_check = frozenset(_redir_check_2)
+
 Operator = group(
     r"\*\*=?",
     r">>=?",
@@ -786,7 +799,7 @@ def detect_encoding(readline):
         except UnicodeDecodeError:
             msg = "invalid or missing encoding declaration"
             if filename is not None:
-                msg = "{} for {!r}".format(msg, filename)
+                msg = f"{msg} for {filename!r}"
             raise SyntaxError(msg)
 
         match = cookie_re.match(line_string)
@@ -800,7 +813,7 @@ def detect_encoding(readline):
             if filename is None:
                 msg = "unknown encoding: " + encoding
             else:
-                msg = "unknown encoding for {!r}: {}".format(filename, encoding)
+                msg = f"unknown encoding for {filename!r}: {encoding}"
             raise SyntaxError(msg)
 
         if bom_found:
@@ -809,7 +822,7 @@ def detect_encoding(readline):
                 if filename is None:
                     msg = "encoding problem: utf-8"
                 else:
-                    msg = "encoding problem for {!r}: utf-8".format(filename)
+                    msg = f"encoding problem for {filename!r}: utf-8"
                 raise SyntaxError(msg)
             encoding += "-sig"
         return encoding
@@ -855,7 +868,7 @@ def tokopen(filename):
         raise
 
 
-def _tokenize(readline, encoding):
+def _tokenize(readline, encoding, tolerant=False):
     lnum = parenlev = continued = 0
     numchars = "0123456789"
     contstr, needcont = "", 0
@@ -886,7 +899,14 @@ def _tokenize(readline, encoding):
 
         if contstr:  # continued string
             if not line:
-                raise TokenError("EOF in multi-line string", strstart)
+                if tolerant:
+                    # return the partial string
+                    yield TokenInfo(
+                        ERRORTOKEN, contstr, strstart, (lnum, end), contline + line
+                    )
+                    break
+                else:
+                    raise TokenError("EOF in multi-line string", strstart)
             endmatch = endprog.match(line)
             if endmatch:
                 pos = end = endmatch.end(0)
@@ -952,7 +972,9 @@ def _tokenize(readline, encoding):
                 indents.append(column)
                 yield TokenInfo(INDENT, line[:pos], (lnum, 0), (lnum, pos), line)
             while column < indents[-1]:
-                if column not in indents:
+                if (
+                    column not in indents and not tolerant
+                ):  # if tolerant, just ignore the error
                     raise IndentationError(
                         "unindent does not match any outer indentation level",
                         ("<tokenize>", lnum, pos, line),
@@ -973,6 +995,9 @@ def _tokenize(readline, encoding):
 
         else:  # continued statement
             if not line:
+                if tolerant:
+                    # no need to raise an error, we're done
+                    break
                 raise TokenError("EOF in multi-line statement", (lnum, 0))
             continued = 0
 
@@ -1110,12 +1135,12 @@ def _tokenize(readline, encoding):
         yield stashed
         stashed = None
 
-    for indent in indents[1:]:  # pop remaining indent levels
+    for _ in indents[1:]:  # pop remaining indent levels
         yield TokenInfo(DEDENT, "", (lnum, 0), (lnum, 0), "")
     yield TokenInfo(ENDMARKER, "", (lnum, 0), (lnum, 0), "")
 
 
-def tokenize(readline):
+def tokenize(readline, tolerant=False):
     """
     The tokenize() generator requires one argument, readline, which
     must be a callable object which provides the same interface as the
@@ -1133,11 +1158,16 @@ def tokenize(readline):
 
     The first token sequence will always be an ENCODING token
     which tells you which encoding was used to decode the bytes stream.
+
+    If ``tolerant`` is True, yield ERRORTOKEN with the erroneous string instead of
+    throwing an exception when encountering an error.
     """
     encoding, consumed = detect_encoding(readline)
     rl_gen = iter(readline, b"")
     empty = itertools.repeat(b"")
-    return _tokenize(itertools.chain(consumed, rl_gen, empty).__next__, encoding)
+    return _tokenize(
+        itertools.chain(consumed, rl_gen, empty).__next__, encoding, tolerant
+    )
 
 
 # An undocumented, backwards compatible, API for all the places in the standard
@@ -1158,7 +1188,7 @@ def tokenize_main():
             args = (filename,) + location + (message,)
             perror("%s:%d:%d: error: %s" % args)
         elif filename:
-            perror("%s: error: %s" % (filename, message))
+            perror(f"{filename}: error: {message}")
         else:
             perror("error: %s" % message)
         sys.exit(1)
